/*
 *	setv4l.c
 *
 *	Copyright 2000 by Jeroen Vreeken (pe1rxq@amsat.org)
 *	This program is published under the GNU Public License version 2.0 or
 *	later, read the file COPYING for more info.
 *
 */
#include "setv4l.h"

void options (void)
{
	printf("setv4l, Version %s\n", VERSION);
	printf("Copyright 2000 by Jeroen Vreeken (pe1rxq@amsat.org)\n");
	printf("Usage:\n");
	printf("setv4l [device] <brightness N> <hue N> <colour N> <contrast N> <whiteness N>\n                <frequency KHZ>\n");
	printf("\n");
}

void print_settings (struct video_picture *picture, struct video_capability *capability)
{
	char temp[255];
	int i=0;
	
	temp[0]=0;
	while (vid_types[i].vidtypenr) {
		if (capability->type & vid_types[i].vidtypenr) {
			sprintf(temp, "%s %s", temp, vid_types[i].description);
		}
		i++;
	}
	printf("capabilities: %s\n", temp);

	printf("min-size: %dx%d \t",
	    capability->minwidth, capability->minheight);
	printf("max-size: %dx%d \t",
	    capability->maxwidth, capability->maxheight);
	printf("channels: %d \t", capability->channels);
	printf("audios: %d\n", capability->audios);
	printf("brightness: %d \t", picture->brightness);
	printf("hue: %d \t", picture->hue);
	printf("colour: %d \t", picture->colour);
	printf("contrast: %d\n", picture->contrast);
	printf("whiteness: %d \t", picture->whiteness);
	printf("depth: %d \t", picture->depth);
	printf("palette: %d\n", picture->palette);
	printf("\n");
}

int parse_arg (struct video_picture *picture, int argc, char **argv)
{
	int i, ret=0;
	
	for (i=2; i<argc; i++) {
		if (!strcmp("brightness", argv[i])) {
			picture->brightness=atoi(argv[i+1]);
			ret=1;
		}
		if (!strcmp("hue", argv[i])) {
			picture->hue=atoi(argv[i+1]);
			ret=1;
		}
		if (!strcmp("colour", argv[i])) {
			picture->colour=atoi(argv[i+1]);
			ret=1;
		}
		if (!strcmp("contrast", argv[i])) {
			picture->contrast=atoi(argv[i+1]);
			ret=1;
		}
		if (!strcmp("whiteness", argv[i])) {
			picture->whiteness=atoi(argv[i+1]);
			ret=1;
		}
	}
	return ret;
}

int parse_arg_signal ( int* frequency, int argc, char **argv)
{
	int i, ret=0;
	
	for (i=2; i<argc; i++) {
		if (!strcmp("frequency", argv[i])) {
			*frequency=atoi(argv[i+1]) * 16 / 1000;
			ret=1;
		}
	}
	return ret;
}

int main (int argc, char **argv)
{
	struct video_capability capability;
	struct video_picture picture;
	int fdvid;
	long int timeout;
	int frequency=0;

	if (argc < 2) {
		options();
		exit(0);
	}
	
	timeout=time(NULL);
	while ((fdvid=open(argv[1], O_RDWR))==-1 &&
	    time(NULL)-timeout < TIMEOUT && errno==EBUSY);
	if (fdvid== -1) {
		perror("Error opening video device");
		exit (-1);
	}
	if (ioctl (fdvid, VIDIOCGCAP, &capability) == -1) {
		perror("Error getting device capabilities");
	}
	printf("device: %s \tname: %s\n", argv[1], capability.name);
	if (ioctl (fdvid, VIDIOCGPICT, &picture) == -1) {
		perror("Error getting picture settings");
	} else {
		if (parse_arg(&picture, argc, argv)) {
			if (ioctl (fdvid, VIDIOCSPICT, &picture) == -1)
				perror("Error setting picture settings");
			if (ioctl (fdvid, VIDIOCGPICT, &picture) == -1)
				perror("Error getting picture settings");
		}
		print_settings(&picture, &capability);
	}

	if (ioctl (fdvid, VIDIOCGFREQ, &frequency) == -1) {
		perror("Error getting signal settings");
	} else {
		if (parse_arg_signal(&frequency, argc, argv)) {
		    if (ioctl (fdvid, VIDIOCSFREQ, &frequency) == -1)
			perror("Error setting signal frequency");
		}
		printf( "current freq: %d khz\n", frequency * 1000 / 16 );
		printf( "\n" );
	}

	close (fdvid);
	exit (0);
}
